

import numpy as np


class StockBinomialTree:
    '''
    Class that implements the binomial tree with stock price
    '''

    def __init__(self, S0=1.0, r=0.05, sigma=0.05, T=1.0, no_steps=100):
        '''
        Constructor
        '''
        self.S0 = S0
        self.r = r
        self.sigma = sigma
        self.T = T
        self.no_steps = no_steps

        self.initializeParameters()
        self.initializeOptionPriceTree()

    def initializeParameters(self):
        self.dt = self.T / self.no_steps
        self.rdt = self.r * self.dt
        self.sigma2dt = self.sigma * self.sigma * self.dt
        self.u = np.exp(np.sqrt(self.sigma2dt))
        self.d = 1. / self.u
        self.p = (np.exp(self.rdt)-self.d) / (self.u-self.d)

    def get_stockprice(self, i, j):
        return self.S0 * self.u ** (i-j) * self.d ** j

    def initializeOptionPriceTree(self):
        optionPriceTree = []
        for i in range(self.no_steps + 1):
            optionPrice = np.zeros(self.no_steps+1)
            optionPriceTree.append(optionPrice)
        self.optionPriceTree = optionPriceTree

    def getPrice(self):
        return self.optionPriceTree[0][0]

    def calculateOptionPriceTree(self):
        raise Exception('Not implemented!')


class EuropeanCallBinomialTree(StockBinomialTree):
    '''
    Binomial Tree for European call option
    '''

    def __init__(self, S0=1.0, X=0.8, r=0.05, sigma=0.05, T=1.0, no_steps=100):
        '''
        Constructor
        '''
        StockBinomialTree.__init__(self, S0=S0, r=r, sigma=sigma, T=T, no_steps=no_steps)
        self.X = X
        self.calculateOptionPriceTree()

    def calculateOptionPriceTree(self):
        for j in range(self.no_steps + 1):
            self.optionPriceTree[self.no_steps][j] = max(self.get_stockprice(self.no_steps, j) - self.X, 0)
        for i in range(self.no_steps - 1, -1, -1):
            for j in range(i + 1):
                self.optionPriceTree[i][j] = np.exp(-self.rdt) * (self.p * self.optionPriceTree[i + 1][j] + (1 - self.p) * self.optionPriceTree[i + 1][j + 1])


class EuropeanPutBinomialTree(StockBinomialTree):
    '''
    Binomial Tree for European put option
    '''

    def __init__(self, S0=1.0, X=0.8, r=0.05, sigma=0.05, T=1.0, no_steps=100):
        '''
        Constructor
        '''
        StockBinomialTree.__init__(self, S0=S0, r=r, sigma=sigma, T=T, no_steps=no_steps)
        self.X = X
        self.calculateOptionPriceTree()

    def calculateOptionPriceTree(self):
        for j in range(self.no_steps + 1):
            self.optionPriceTree[self.no_steps][j] = max(self.X - self.get_stockprice(self.no_steps, j), 0)
        for i in range(self.no_steps - 1, -1, -1):
            for j in range(i + 1):
                self.optionPriceTree[i][j] = np.exp(-self.rdt) * (self.p * self.optionPriceTree[i + 1][j] + (1 - self.p) * self.optionPriceTree[i + 1][j + 1])


class AmericanCallBinomialTree(StockBinomialTree):
    '''
    Binomial Tree for American call option
    '''

    def __init__(self, S0=1.0, X=0.8, r=0.05, sigma=0.05, T=1.0, no_steps=100):
        '''
        Constructor
        '''
        StockBinomialTree.__init__(self, S0=S0, r=r, sigma=sigma, T=T, no_steps=no_steps)
        self.X = X

        self.calculateOptionPriceTree()

    def calculateOptionPriceTree(self):
        for j in range(self.no_steps + 1):
            self.optionPriceTree[self.no_steps][j] = max(self.get_stockprice(self.no_steps, j) - self.X, 0)
        for i in range(self.no_steps - 1, -1, -1):
            for j in range(i + 1):
                imValue = self.get_stockprice(i, j) - self.X
                euroCallValue = np.exp(-self.rdt) * (self.p * self.optionPriceTree[i + 1][j] + (1 - self.p) * self.optionPriceTree[i + 1][j + 1])
                self.optionPriceTree[i][j] = max(imValue, euroCallValue)


class AmericanPutBinomialTree(StockBinomialTree):
    '''
    Binomial Tree for American put option
    '''

    def __init__(self, S0=1.0, X=0.8, r=0.05, sigma=0.05, T=1.0, no_steps=100):
        '''
        Constructor
        '''
        StockBinomialTree.__init__(self, S0=S0, r=r, sigma=sigma, T=T, no_steps=no_steps)
        self.X = X

        self.calculateOptionPriceTree()

    def calculateOptionPriceTree(self):
        for j in range(self.no_steps + 1):
            self.optionPriceTree[self.no_steps][j] = max(self.X - self.get_stockprice(self.no_steps, j), 0)
        for i in range(self.no_steps - 1, -1, -1):
            for j in range(i + 1):
                imValue = self.X - self.get_stockprice(i, j)
                euroPutValue = np.exp(-self.rdt) * (self.p * self.optionPriceTree[i + 1][j] + (1 - self.p) * self.optionPriceTree[i + 1][j + 1])
                self.optionPriceTree[i][j] = max(imValue, euroPutValue)


import math

import numpy as np
from numba import jit

from ..util.derivatives import numerical_gradients as ng

@jit
def sigmoid_prob(x, A, B):
    return 1./(1+math.exp(A*x+B))

@jit
def label(f, threshold=0.):
    if f == threshold:
        return 0
    else:
        return 1 if f > threshold else -1
        

@jit
def entropy_error_fcn(scores, A, B):
    fcnval = 0.
    for score in scores:
        y = label(score)
        t = (y+1) / 2
        p = sigmoid_prob(score, A, B)
        if p > 0 and p < 1:
            fcnval += - t*math.log(p) - (1-t)*math.log(1-p)
        elif p == 0.:
            fcnval += - (1-t)*math.log(1-p)            
        elif p == 1.:
            fcnval += - t*math.log(p)            
    return fcnval


@jit
def gradient_search_min_entropy_error_fcn(scores, tol=1e-1, sigma=0.1):
    max_score = max(scores)
    min_score = min(scores)
    initA = -30. / (max_score - min_score)
    initB = -0.5*(max_score+min_score) * initA
    
    A = initA
    B = initB
    curr_fcn_val = entropy_error_fcn(scores, A, B)
    converged = False
    while not converged:
        dfA, dfB = ng.tensor_gradient(lambda X: entropy_error_fcn(scores, X[0], X[1]),
                                      np.array([A, B]),
                                      tol=tol)
        new_fcn_val = entropy_error_fcn(scores, A-sigma*dfA, B-sigma*dfB)
        if abs(new_fcn_val-curr_fcn_val) < tol:
            converged=True
        curr_fcn_val = new_fcn_val
        A = A-sigma*dfA
        B = B-sigma*dfB

    return A, B




from itertools import product

import unittest
import numpy as np

import mogu


class test_random(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_sample_correlted_gaussian(self):
        mean = np.array([2., -1.5])
        covmat = np.array([[2.3, -1.1], [-1.1, 1.25]])
        nb_samples = 100000

        # sampling
        rndmat = mogu.random.multivariate.multivar_gaussian(mean, covmat, size=nb_samples)

        # calculating statistics
        sampled_mean = np.mean(rndmat, axis=1)
        sampled_cov = np.cov(rndmat)

        # checking
        for i in range(len(mean)):
            self.assertAlmostEqual(sampled_mean[i],
                                   mean[i],
                                   delta=1.96*np.sqrt(covmat[i, i]/nb_samples))

        # ref: https://stats.stackexchange.com/questions/287144/variance-of-a-sample-covariance-for-normal-variables
        corrcoef = covmat[0, 1]/np.sqrt(covmat[0, 0]*covmat[1, 1])
        cov_of_cov = np.array([[2*covmat[0, 0]*covmat[0, 0], (1+corrcoef*corrcoef)*covmat[0, 0]*covmat[1, 1]],
                               [(1+corrcoef*corrcoef)*covmat[0, 0]*covmat[1, 1], 2*covmat[1, 1]*covmat[1, 1]]])
        cov_of_cov /= np.sqrt(nb_samples-1.)
        for i, j in product(range(len(mean)), range(len(mean))):
            self.assertAlmostEqual(sampled_cov[i, j],
                                   covmat[i, j],
                                   delta=1.96*cov_of_cov[i, j])


if __name__ == '__main__':
    unittest.main()


from operator import add
from math import sqrt

from numba import jit

golden_ratio = (sqrt(5) - 1) * 0.5

@jit(cache=True)
def sumSquaresErrors(xlist, ylist, alpha):
    if len(xlist) != len(ylist):
        return None
    return reduce(add, map(lambda x, y: (x ** alpha - y) ** 2, xlist, ylist))

@jit(cache=True)
def goldenSearchMin(func, x1, x2, tol=1e-5):
    if abs(x1 - x2) < tol:
        return x1
    xl = x1 + (x2 - x1) * (1 - golden_ratio)
    xr = x1 + (x2 - x1) * golden_ratio
    fl = func(xl)
    fr = func(xr)
    return goldenSearchMin(func, x1, xr, tol=tol) if fl <= fr else goldenSearchMin(func, xl, x2, tol=tol)





import numpy as np

from binomialtree import binomialtree as bt

def compute_bintree_parameters(r, sigma, T, nbsteps):
    dt = T / float(nbsteps)
    rdt = r * dt
    sigma2dt = sigma * sigma * dt
    u = np.exp(np.sqrt(sigma2dt))
    d = 1. / u
    p = (np.exp(rdt) - d) / (u - d)

    return {'dt': dt, 'rdt': rdt, 'sigma2dt': sigma2dt, 'u': u, 'd': d, 'p': p}

def option_pricing(S, X, r, sigma, T, nbsteps, f90func):
    parameters = compute_bintree_parameters(r, sigma, T, nbsteps)
    return f90func(S, X, parameters['rdt'], parameters['p'], parameters['u'], parameters['d'], nbsteps)

def eurocall_price(S, X, r, sigma, T, nbsteps):
    return option_pricing(S, X, r, sigma, T, nbsteps, bt.eurocall)

def europut_price(S, X, r, sigma, T, nbsteps):
    return option_pricing(S, X, r, sigma, T, nbsteps, bt.europut)

def amcall_price(S, X, r, sigma, T, nbsteps):
    return option_pricing(S, X, r, sigma, T, nbsteps, bt.amcall)

def amput_price(S, X, r, sigma, T, nbsteps):
    return option_pricing(S, X, r, sigma, T, nbsteps, bt.amput)




import numpy as np

def multivar_gaussian(mean, covariance, size=None):
    # getting shapes
    n = mean.shape[0]
    covmat_shape = covariance.shape

    # number of sampled data
    size = size if not (size is None) else 1

    # checking shape compliance
    assert len(covmat_shape) == 2
    assert covmat_shape[0] == n
    assert covmat_shape[1] == n
    assert np.allclose(covariance, covariance.T, atol=1e-4)

    # diagonalizing covariance matrix
    diagvar, eigvecs = np.linalg.eig(covariance)
    transmat = np.linalg.inv(eigvecs)

    # sampling
    rndmatrix = np.random.normal(size=size*n).reshape((n, size))
    # scale by diagonal variance
    rndmatrix *= np.array([np.sqrt(diagvar)]).T
    # linear transform
    rndmatrix = np.matmul(transmat.T, rndmatrix)
    # adding the mean
    rndmatrix += np.array([mean]).T

    return rndmatrix



# rndmat = multivar_gaussian(np.array([0, 1]), np.array([[1.1, 0.5], [0.5, 1.5]]), size=10000)
# np.mean(rndmat, axis=1)
# np.cov(rndmat)



from itertools import product

import numpy as np
from numba import jit

# abstract tensor derivatives
@jit(cache=True)
def abstract_tensor_derivative_element(tensor_derivative_expression, fcn, X, indices, dx=None, tol=1e-8):
    if dx == None:
        tdx = min(0.1, abs(0.1 * X[indices])) if X[indices] != 0. else 0.1
        df_old = abstract_tensor_derivative_element(tensor_derivative_expression, fcn, X, indices, dx=tdx)
        df_new = abstract_tensor_derivative_element(tensor_derivative_expression, fcn, X, indices, dx=tdx * 0.1)
        while abs(df_new - df_old) > tol:
            df_old = df_new
            tdx *= 0.1
            df_new = abstract_tensor_derivative_element(tensor_derivative_expression, fcn, X, indices, dx=tdx)
        return df_new
    else:
        return tensor_derivative_expression(fcn, X, indices, dx)

@jit(cache=True)
def abstract_derivative_gradient(tensor_derivative_expression, fcn, X, *args, **kwargs):
    dfval = np.zeros(X.shape)
    for indices in product(*map(range, X.shape)):
        dfval[indices] = abstract_tensor_derivative_element(tensor_derivative_expression, fcn, X, indices, *args, **kwargs)
    return dfval

@jit(cache=True)
def grad_element(fcn, X, indices, dx):
    dX = np.zeros(X.shape)
    dX[indices] = dx
    return (fcn(X + dX) - fcn(X - dX)) / (2 * dx)

@jit(cache=True)
def divgrad_element(fcn, X, indices, dx):
    dX = np.zeros(X.shape)
    dX[indices] = dx
    return (fcn(X + dX) - 2 * fcn(X) + fcn(X - dX)) / (dx * dx)

# gradient
def tensor_gradient_element(fcn, X, indices, dx=None, tol=1e-8):
    return abstract_tensor_derivative_element(grad_element, fcn, X, indices, dx=dx, tol=tol)

def tensor_gradient(fcn, X, *args, **kwargs):
    return abstract_derivative_gradient(grad_element, fcn, X, *args, **kwargs)

# divergence of gradient (second derivatives)
def tensor_divgrad_element(fcn, X, indices, dx=None, tol=1e-12):
    return abstract_tensor_derivative_element(divgrad_element, fcn, X, indices, dx=dx, tol=tol)

def tensor_divgrad(fcn, X, *args, **kwargs):
    return abstract_derivative_gradient(divgrad_element, fcn, X, *args, **kwargs)



import numpy as np

def gini_coef(wealths):
    cum_wealths = np.cumsum(sorted(np.append(wealths, 0)))
    sum_wealths = cum_wealths[-1]
    xarray = np.array(range(0, len(cum_wealths))) / np.float(len(cum_wealths)-1)
    yarray = cum_wealths / sum_wealths
    B = np.trapz(yarray, x=xarray)
    A = 0.5 - B
    return A / (A+B)



import warnings

def deprecated(func):
    """This is a decorator which can be used to mark functions
    as deprecated. It will result in a warning being emmitted
    when the function is used."""
    def newFunc(*args, **kwargs):
        warnings.warn("Call to deprecated function %s." % func.__name__,
                      category=DeprecationWarning)
        return func(*args, **kwargs)
    newFunc.__name__ = func.__name__
    newFunc.__doc__ = func.__doc__
    newFunc.__dict__.update(func.__dict__)
    return newFunc



from . import util

from . import assrule
from . import finance
from . import fit
from . import netflow
from . import voterank
from . import dynprog
from . import topology
from . import probxwalk
from . import random

from .netflow import simvoltage

from . import econ




import os
import unittest

import numpy as np
import pandas as pd
from mogu.netflow import simvoltage

class test_SocialNetwork(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_socialnetwork(self):
        nodes = ['Stephen', 'John', 'Mary',
                 'Joshua',
                 'Abigail', 'Andrew', 'Jacob', 'Melanie',
                 'Shirley', 'Zoe', 'Wallace', 'Susan',
                 'Urban']
        edges = [('Stephen', 'Jacob', 1),
                 ('Jacob', 'Stephen', 1),
                 ('Stephen', 'Abigail', 1),
                 ('Abigail', 'Stephen', 1),
                 ('Stephen', 'Andrew', 1),
                 ('Andrew', 'Stephen', 1),
                 ('Andrew', 'Abigail', 1),
                 ('Abigail', 'Andrew', 1),
                 ('John', 'Stephen', 1),
                 ('Andrew', 'John', 0.4),
                 ('John', 'Andrew', 0.6),
                 ('Abigail', 'John', 1),
                 ('John', 'Abigail', 1),
                 ('John', 'Mary', 1),
                 ('Mary', 'John', 0.9),
                 ('John', 'Joshua', 1),
                 ('Joshua', 'John', 1),
                 ('John', 'Jacob', 1),
                 ('Jacob', 'John', 1),
                 ('Abigail', 'Jacob', 1),
                 ('Jacob', 'Abigail', 1),
                 ('Jacob', 'Andrew', 1),
                 ('Andrew', 'Jacob', 1),
                 ('Shirley', 'Stephen', 1),
                 ('Stephen', 'Shirley', 1),
                 ('Melanie', 'Stephen', 1),
                 ('Stephen', 'Melanie', 1),
                 ('Melanie', 'Shirley', 1),
                 ('Shirley', 'Urban', 0.2),
                 ('Urban', 'Shirley', 0.21),
                 ('Susan', 'Shirley', 1),
                 ('Shirley', 'Susan', 1),
                 ('Shirley', 'Zoe', 1),
                 ('Zoe', 'Shirley', 1),
                 ('Shirley', 'Wallace', 1),
                 ('Wallace', 'Shirley', 1),
                 ('Zoe', 'Wallace', 1)]
        wn1 = simvoltage.SocialNetworkSimVoltage(nodes=nodes, edges=edges, precalculated_distance=True)

        THIS_DIR = os.path.dirname(os.path.abspath(__file__))
        testresults = pd.read_csv(os.path.join(THIS_DIR, 'socialnetworkranks.csv'),
                                  header=None,
                                  names=['name1', 'name2', 'resistance'],
                                  dtype={'name1': str, 'name2': str, 'resistance': np.float})

        for _, row in testresults.iterrows():
            name1 = row['name1']
            name2 = row['name2']
            resistance = row['resistance']
            self.assertAlmostEqual(wn1.getResistance(name1, name2), resistance, places=3)
            print('%s\t%s : %.4f passed.' % (name1, name2, resistance))

    def test_resistdist(self):
        obj = simvoltage.GraphResistanceDistance()
        self.assertAlmostEqual(obj.getResistance('Stephen', 'Sinnie'), 0.6666666666666667, places=4)
        self.assertAlmostEqual(obj.getResistance('Elaine', 'Sinnie'), 0.6666666666666667, places=4)
        self.assertAlmostEqual(obj.getResistance('Elaine', 'Stephen'), 0.6666666666666667, places=4)

if __name__ == '__main__':
    unittest.main()



import unittest
import numpy as np

from mogu.topology import SimplicialComplex, AlphaComplex

# Simplicial Complex
# ring
ring1d_sc = [(i, i+1) for i in range(10)] + [(10,0)]
# cylinder with hole
cylinder_sc = [(0, 1, 2), (1, 2, 3), (2, 3, 0), (3, 0, 1)]
# torus (https://page.mi.fu-berlin.de/rote/Papers/pdf/Computational+topology:+an+introduction.pdf)
torus_sc = [(1,2,4), (4,2,5), (2,3,5), (3,5,6), (5,6,1), (1,6,2), (6,7,2), (7,3,2),
            (1,3,4), (3,4,6), (4,6,7), (4,5,7), (5,7,1), (7,3,1)]
# sphere
twosphere_c = [(1,3,4), (1,2,3), (2,3,4), (1,2,4)]

# Alpha Complex
# circular ring
ring = np.array([np.random.normal(loc=1, scale=0.01)*np.array([np.cos(t), np.sin(t)]) for t in np.linspace(0, 2*np.pi, 101)])[1:]
# spherical ball
sphere = np.array([[np.sin(theta)*np.cos(phi), np.sin(theta)*np.sin(phi), np.cos(theta)] for theta in np.linspace(0.02, np.pi, 51) for phi in np.linspace(0.02, 2*np.pi, 51)])

class test_simcomplex(unittest.TestCase):
    def setUp(self):
        self.sc1 = SimplicialComplex(simplices=[(1, 2), (2, 3), (1, 2, 3)])
        self.sc2 = AlphaComplex(ring, 0.05)
        self.ring1d = SimplicialComplex(simplices=ring1d_sc)
        self.cylinder = SimplicialComplex(simplices=cylinder_sc)
        self.torus = SimplicialComplex(simplices=torus_sc)
        self.twosphere = SimplicialComplex(simplices=twosphere_c)

    def tearDown(self):
        pass

    def test_ring1d(self):
        self.assertEqual(self.ring1d.betti_number(0), 1)
        self.assertEqual(self.ring1d.betti_number(1), 1)
        self.assertEqual(self.ring1d.betti_number(2), 0)

    def test_cylinder(self):
        self.assertEqual(self.cylinder.betti_number(0), 1)
        self.assertEqual(self.cylinder.betti_number(1), 0)
        self.assertEqual(self.cylinder.betti_number(2), 1)

    def test_torus(self):
        self.assertEqual(self.torus.betti_number(0), 1)
        self.assertEqual(self.torus.betti_number(1), 2)
        self.assertEqual(self.torus.betti_number(2), 1)

    def test_twosphere(self):
        self.assertEqual(self.twosphere.betti_number(0), 1)
        self.assertEqual(self.twosphere.betti_number(1), 0)
        self.assertEqual(self.twosphere.betti_number(2), 1)

    def test_sc1(self):
        self.assertEqual(self.sc1.betti_number(0), 1)
        self.assertEqual(self.sc1.betti_number(1), 0)

    def test_sc2(self):
        self.assertEqual(self.sc2.betti_number(0), 1)
        self.assertEqual(self.sc2.betti_number(1), 1)
        self.assertEqual(self.sc2.betti_number(2), 0)

    # def test_sc2_sparse(self):
    #     self.assertEqual(self.sc2.betti_number(0, eps=0.001), 1)
    #     self.assertEqual(self.sc2.betti_number(1, eps=0.001), 1)
    #     self.assertEqual(self.sc2.betti_number(2, eps=0.001), 0)



if __name__ == '__main__':
    unittest.main()


import numpy as np
from scipy.stats import norm

def diffscore(npos, nneg):
    return npos-nneg

def fracscore(npos, nneg):
    return npos/np.array(npos+nneg, dtype=float)

def wilsonscore(npos, nneg, alpha=0.1):
    num = npos+nneg
    p = fracscore(npos, nneg)
    z = norm.ppf(1.-0.5*alpha)
    denominator = 1+z*z/num
    numerator = p+0.5*z*z/num-z*np.sqrt((p*(1-p)+0.25*z*z/num)/num)
    return numerator/denominator



import numpy as np
cimport numpy as np

def damerau_levenshtein(str word1, str word2):
    cdef int len1 = len(word1)
    cdef int len2 = len(word2)

    cdef int i, j
    cdef int cost, delcost, inscost, subcost, score

    cdef int[:, :] matrix = np.zeros((len1+1, len2+1), dtype=np.intc)

    for i in range(len1+1):
        matrix[i][0] = i
    for j in range(len2+1):
        matrix[0][j] = j

    for i in range(len1+1):
        for j in range(len2+1):
            cost = 0
            if i>0 and j>0 and (word1[i-1]!=word2[j-1]):
                cost = 1
            delcost = matrix[i-1][j] + 1
            inscost = matrix[i][j-1] + 1
            subcost = matrix[i-1][j-1] + cost
            score = min(min(delcost, inscost), subcost)
            if ((i>1) & (j>1) & (word1[i-1]==word2[j-2]) & (word1[i-2]==word2[j-1])):
                score = min(score, matrix[i-2][j-2]+cost)
            matrix[i][j] = score

    return matrix[len1][len2]




import mogu.probxwalk as xwalk
import unittest


class test_probxwalk(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_uniformweights(self):
        example_mapping = {'a': 'A',
                           'b': 'B',
                           'ab': ['A', 'B'],
                           'num': [0, 1, 2]}
        crosswalk = xwalk.compute_uniformweighted_xwalk(example_mapping)
        self.assertEqual(len(crosswalk), 4)
        self.assertAlmostEqual(crosswalk['a']['A'], 1.0, delta=0.0001)
        self.assertAlmostEqual(crosswalk['ab']['A'], 0.5, delta=0.0001)
        self.assertAlmostEqual(crosswalk['ab']['B'], 0.5, delta=0.0001)
        self.assertAlmostEqual(crosswalk['b']['B'], 1.0, delta=0.0001)
        self.assertAlmostEqual(crosswalk['num'][0], 1./3., delta=0.0001)
        self.assertAlmostEqual(crosswalk['num'][1], 1. / 3., delta=0.0001)
        self.assertAlmostEqual(crosswalk['num'][2], 1. / 3., delta=0.0001)

    def test_double_xwalk(self):
        xwalk1 = {'a': {'A': 1}, 'b': {'B': 0.3, 'C': 0.7}}
        xwalk2 = {'A': {1: 1}, 'B': {2: 0.5, 3: 0.5}, 'C': {4: 1}}
        crosswalk = xwalk.compute_resultant_xwalk(xwalk1, xwalk2)
        self.assertAlmostEqual(crosswalk['a'][1], 1, delta=0.0001)
        self.assertAlmostEqual(crosswalk['b'][2], 0.15, delta=0.0001)
        self.assertAlmostEqual(crosswalk['b'][3], 0.15, delta=0.0001)
        self.assertAlmostEqual(crosswalk['b'][4], 0.7, delta=0.0001)


if __name__ == '__main__':
    unittest.main()




import unittest

import mogu

class TestFuzzyLogic(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass


    def test_similarity(self):
        self.assertEqual(mogu.dynprog.damerau_levenshtein('debug', 'deubg'), 1)
        self.assertEqual(mogu.dynprog.damerau_levenshtein('intrdependence', 'interdpeendencae'), 3)
        self.assertEqual(mogu.dynprog.longest_common_prefix('debug', 'debuag'), 4)

    def test_transposition(self):
        self.assertEqual(mogu.dynprog.damerau_levenshtein('independent', 'indeepndent'), 1)
        self.assertEqual(mogu.dynprog.damerau_levenshtein('providence', 'porvidecne'), 2)

    def test_insertion(self):
        self.assertEqual(mogu.dynprog.damerau_levenshtein('algorithm', 'algorithms'), 1)
        self.assertEqual(mogu.dynprog.damerau_levenshtein('algorithm', 'algoarithmm'), 2)

    def test_deletion(self):
        self.assertEqual(mogu.dynprog.damerau_levenshtein('algorithm', 'algoithm'), 1)
        self.assertEqual(mogu.dynprog.damerau_levenshtein('algorithm', 'algorith'), 1)
        self.assertEqual(mogu.dynprog.damerau_levenshtein('algorithm', 'algrihm'), 2)

    def test_correct(self):
        self.assertEqual(mogu.dynprog.damerau_levenshtein('python', 'python'), 0)
        self.assertEqual(mogu.dynprog.damerau_levenshtein('sosad', 'sosad'), 0)


if __name__ == '__main__':
    unittest.main()




import numpy as np
from collections import defaultdict


def compute_uniformweighted_xwalk(mappings):
    xwalk_dict = {}
    for key in mappings:
        if type(mappings[key]) is not list:
            xwalk_dict[key] = {mappings[key]: 1.0}
        else:
            xwalk_dict[key] = {target: 1./len(mappings[key]) for target in mappings[key]}
    return xwalk_dict


def compute_resultant_xwalk(xwalk1, xwalk2):
    result_xwalk_dict = {}
    for key in xwalk1:
        result_xwalk_dict[key] = defaultdict(lambda : 0)
        for intermediate_key in xwalk1[key]:
            weight = xwalk1[key][intermediate_key]
            for target in xwalk2[intermediate_key]:
                result_xwalk_dict[key][target] += weight*xwalk2[intermediate_key][target]
        result_xwalk_dict[key] = dict(result_xwalk_dict[key])
    return result_xwalk_dict




# must import thisfirst. Ref: # must import this. Ref: https://stackoverflow.com/questions/7932028/setup-py-for-packages-that-depend-on-both-cython-and-f2py?rq=1
from setuptools import setup

import numpy as np
from numpy.distutils.core import setup, Extension

# https://stackoverflow.com/questions/7932028/setup-py-for-packages-that-depend-on-both-cython-and-f2py
# how to include f2py and cython at the same time

try:
    from Cython.Build import cythonize
    dynprog_ext_modules = cythonize(['mogu/dynprog/dldist.pyx', 'mogu/dynprog/lcp.pyx'])
except ImportError:
    dynprog_ext_modules = [Extension('mogu.dynprog.dldist', ['mogu/dynprog/dldist.c']),
                           Extension('mogu.dynprog.lcp', ['mogu/dynprog/lcp.c'])]


def readme():
    with open('README.md') as f:
        return f.read()


setup(name='mogu',
      version="0.3.0",
      description="Collection of Simple Numerical Routines",
      long_description="Collection of simple numerical routines, independent of each other",
      classifiers=[
          "Topic :: Scientific/Engineering :: Information Analysis",
          "Topic :: Scientific/Engineering :: Mathematics",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3.5",
          "Programming Language :: Python :: 3.6",
          "Programming Language :: Python :: 3.7",
          "Programming Language :: Fortran",
          "Programming Language :: Cython",
          "Programming Language :: C",
          "License :: OSI Approved :: MIT License",
      ],
      keywords="mogu numerics computation",
      url="https://github.com/stephenhky/MoguNumerics",
      author="Kwan-Yuet Ho",
      author_email="stephenhky@yahoo.com.hk",
      license='MIT',
      packages=['mogu',
                'mogu.fit',
                'mogu.util',
                'mogu.util.derivatives',
                'mogu.assrule',
                'mogu.voterank',
                'mogu.finance',
                'mogu.finance.binomial',
                'mogu.dynprog',
                'mogu.probxwalk',
                'mogu.random',
                'mogu.econ'],
      package_data={'mogu': ['finance/binomial/*.f90', 'finance/binomial/*.pyf', 'dynprog/*.pyx'],
                    'test': ['*.csv']},
      setup_requires=['numpy', 'Cython'],
      install_requires=[
          'Cython', 'numpy', 'scipy', 'numba', 'networkx>=2.0', 'graphflow>=0.1.1', 'mogutda>=0.1.1',
      ],
      tests_require=[
          'unittest2', 'pandas',
      ],
      scripts=['bin/concatenate_dict', 'bin/mogu_minerule', 'bin/price_option'],
      include_dirs=[np.get_include()],
      ext_modules = [Extension( 'binomialtree',
                                sources=['mogu/finance/binomial/binomialtree.f90',
                                         'mogu/finance/binomial/binomialtree.pyf'] ),]
                    +dynprog_ext_modules,
      include_package_data=True,
      test_suite="test",
      zip_safe=False)





from operator import and_
from itertools import combinations
import csv

from numba import jit

class AprioriAssociationRule:
    def __init__(self, inputfile):
        self.transactions = []
        self.itemSet = set([])
        inf = open(inputfile, 'rb')
        reader = csv.reader(inf)
        for elements in reader:
            if len(elements)>0:
                self.transactions.append(filter(lambda item: len(item)>0, elements))
                for element in elements:
                    self.itemSet.add(element)
        inf.close()
        self.toRetItems = {}
        self.associationRules = []

    def getSupport(self, itemcomb):
        if type(itemcomb) != frozenset:
            itemcomb = frozenset([itemcomb])
        within_transaction = lambda transaction: reduce(and_, [(item in transaction) for item in itemcomb])
        count = len(filter(within_transaction, self.transactions))
        return float(count)/float(len(self.transactions))

    def runApriori(self, minSupport=0.15, minConfidence=0.6):
        itemCombSupports = filter(lambda freqpair: freqpair[1]>=minSupport,
                                  map(lambda item: (frozenset([item]), self.getSupport(item)), self.itemSet))
        currentLset = set(map(lambda freqpair: freqpair[0], itemCombSupports))
        k = 2
        while len(currentLset)>0:
            currentCset = set([i.union(j) for i in currentLset for j in currentLset if len(i.union(j))==k])
            currentItemCombSupports = filter(lambda freqpair: freqpair[1]>=minSupport,
                                             map(lambda item: (item, self.getSupport(item)), currentCset))
            currentLset = set(map(lambda freqpair: freqpair[0], currentItemCombSupports))
            itemCombSupports.extend(currentItemCombSupports)
            k += 1
        for key, supportVal in itemCombSupports:
            self.toRetItems[key] = supportVal
        self.calculateAssociationRules(minConfidence=minConfidence)

    def calculateAssociationRules(self, minConfidence=0.6):
        for key in self.toRetItems:
            subsets = [frozenset(item) for k in range(1, len(key)) for item in combinations(key, k)]
            for subset in subsets:
                confidence = self.toRetItems[key] / self.toRetItems[subset]
                if confidence > minConfidence:
                    self.associationRules.append([subset, key-subset, confidence])





import numpy as np
cimport numpy as np

def longest_common_prefix(str word1, str word2):
    cdef int len1 = len(word1)
    cdef int len2 = len(word2)

    cdef int lcp = 0
    cdef int i

    for i in range(min(len1, len2)):
        if word1[i] == word2[i]:
            lcp += 1
        else:
            break

    return lcp





import unittest

from mogu.finance.binomial import eurocall_price, europut_price, amcall_price, amput_price

class test_finance(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testEuroCall(self):
        self.assertAlmostEquals(eurocall_price(100.0, 110.0, 0.01, 0.01, 10.0, 10000),
                                1.4968152046203613,
                                delta=1e-4)

    def testEuroPut(self):
        self.assertAlmostEquals(europut_price(100.0, 80.0, 0.05, 0.05, 10.0, 10000),
                                5.284386951732304e-06,
                                delta=1e-6)

    def testAmCall(self):
        self.assertAlmostEquals(amcall_price(100.0, 110.0, 0.05, 0.05, 10.0, 10000),
                                33.303010096394594,
                                delta=1e-3)

    def testAmPut(self):
        self.assertAlmostEquals(amput_price(100.0, 80.0, 0.05, 0.05, 10.0, 10000),
                                9.38288088691752e-05,
                                delta=1e-6)


if __name__ == '__main__':
    unittest.main()